###################
##R - Some Basics##
###################


###################
#R as a Calculator#
###################
#Addition: +
#Subtraction: -
#Multiplication: *
#Division: /
#Exponentiation: ^


# Addition
5 + 5 

#Subtraction
6 - 5 

# A multiplication
3 * 5

# A division
(5 + 5) / 2 

# Exponentiation
2^5


###########################
####VARIABLE ASSIGNMENT####
###########################
#A basic concept in (statistical) programming is called a variable. 
#A variable allows you to store a value (e.g. 4) or an object (e.g. a function description) in R. 
#You can then later use this variable's name to easily access the value or the object that is stored within this variable. 

#Here is a simple Example
X <- 20
Y <- 5
#Printing Variables
X
Y


#Assign the value of 20 to x
x = 20

#Assign the value of 5 to y
y = 5

#####Look in your global environment of the top right. You can see that you have two values stored (x and y)

#Print the x and y variables (note: r is case sensitive)
x
y

################
###Data Types###
################
#Decimal values like 4.5 are called numerics.
#Natural numbers like 4 are called integers. Integers are also numerics.
#Boolean values (TRUE or FALSE) are called logical.
#Text (or string) values are called characters.

#How can I examine my data to determine which category it falls into?
#Class function --> class()

# Create my_numeric to be equal to 42
my_numeric <- 42

# Create my_character to be "universe"
my_character <- "universe"

# Create my_logical to be FALSE
my_logical <- FALSE

#Check to see what data type your object is using the class function
class(my_numeric)
class(my_character)
class(my_logical)


#################
#####VECTORS#####
#################
#A vector is a sequence of data elements of the same basic type
#Members in a vector are officially called components
#here is an example of a vector containing three numeric values --> c(2,4,6)
#A vector containing three character strings --> c("aa", "bb", "cc") ??? NOTE THE QUOTATION MARKS
#A vector with three logical values --> c(TRUE, FALSE, TRUE)

#A vector containing three numeric values (2,4,6) called numbers
numbers = c(2,4,6)

#A vector containing three character strings (aaaaaa, aaabbbaa, aacccaa) called alpha
#DON'T FORGET THE QUOTATION MARKS!!!
alpha = c("aa", "bb", "cc")

#A vector with three logical values (TRUE, FALSE, TRUE) TRUE_FALSE
TRUE_FALSE = c(TRUE, FALSE, TRUE)

#Print the Vectors you just created
numbers
alpha
TRUE_FALSE

#Check to see what type of data type your object is using the class function
class(numbers)
class(alpha)
class(TRUE_FALSE)


###################################
######PLAYING WITH ASSIGNMENT######
###################################
q1 <- c(10.4, 5.6, 3.1, 6.4, 21.7)
assign("q2", c(10.4, 5.6, 3.1, 6.4, 21.7))
c(10.4, 5.6, 3.1, 6.4, 21.7) -> q3

#Print the Vectors you just created
q1
q2
q3

#Divide all the numbers in the vector by 2
numbers2 <- numbers/2

#Print numbers2
numbers2

###################################
######LISTS --> THE BASICS#####
###################################

Lst <- list(name="Fred", wife="Mary", no.children=3,
            child.ages=c(4,7,9))

#Print the list above
Lst

#Aspects of the list
Lst$name
Lst[[1]]





#Great work!!! Now back to PowerPoint for a few minutes










#Let's clear our "global environment" before we get started as we do not need any of the values or data that are currently stored
#Click the session drop down at the top of the screen and click "clear workspace". A prompt window will appear --> click yes


###############################################
##########Installing R Packages################
###############################################
install.packages("dplyr")
install.packages("data.table")
install.packages("ggplot2")
install.packages("readxl")
install.packages("tidyverse")
#install.packages("dsr")

###Load needed packages
library(data.table)
library(tidyverse)
library(readxl)
#library(dsr)

###############Set your working directory###################
#THIS IS THE FOLDER WHERE YOU ARE STORING ALL OF YOUR DATA##
############################################################

#Let's see what our currently working directory is?
getwd()# what is your current working directory

#Let's all agree on a place that we will all store our data --> might be best to store it in our "my documents" folder

#Set your working Directory
setwd("C:\\Users\\xxxx\\OneDrive - Co-operators\\Desktop\\Brock\\Datathon - R Workshop\\2019")

################################
#Importing Data and Exploration#
################################
#There are various functions and packages that can be used and are readily available. However, we are going to stick
#with the ones listed below for simplicity.
#Text --> read.table
#CSV --> read.csv  ;  read.table
#Excel --> read_excel


#######IMPORTING THE TUBERCULOSIS DATA############
#Text
table4 <- read.table("Tidy_table4.txt", header = TRUE, sep = ";")#semi colon
table5 <- read.table("Tidy_table5.txt", header = TRUE, sep = "\t")# tab

#CSV
table1 <- read.csv("Tidy_table1.csv", header = TRUE, sep = ",", stringsAsFactors = FALSE)#import character variables as character
table2 <- read.table("Tidy_table2.csv", header = TRUE, sep = ",")# import character variables as factor

#Excel
table3 <- read_xlsx("TidyData.xlsx", sheet = "table3")#sheet by name
table3 <- read_xlsx("TidyData.xlsx", sheet = 3)#sheet by number


#######################
##Examining your Data##
#######################
# list objects in the Global environment
ls()

# list the variables in table1
names(table1)

# list the structure of the tables
str(table1)
str(table2)
str(table3)
str(table4)
str(table5)


# list levels of factor v1 in mydata
levels(table2$country)
levels(table2$key)
levels(table4$country)

# dimensions of an object
#to read as rows and columns
dim(table1)
dim(table2)
dim(table3)
dim(table4)
dim(table5)

# class of an object (numeric, matrix, data frame, etc)
class(table1)
class(table2)
class(table3)
class(table4)
class(table5)

# print your data
table1
table2
table3
table4
table5

# print first 10 rows of table2
head(table2, n=10)

# print last 5 rows of table1
tail(table1, n=5) 

#########Examining table1

#Summary Statistics
summary(table1)# summary statistics for all data in the table1 data frame

summary(table1$population)# summary statistics for the population variable in the table1 data frame
summary(table1$cases)# summary statistics for the cases variable in the table1 data frame


#Turn off scientific Notation
options(scipen=999)

mean(table1$population)# examine the mean of the population column in the table1 data frame

range(table1$population)# examine the mean of the population column in the table1 data frame


# View your data
View(table1)
View(table2)
View(table3)
View(table4)
View(table5)


#Let's re-import our data again so that we are all working with the original data
#First, let's clean our working environment
#We can do so using the session drop down menu and choosing "Clear Workspace"
#OR
#we can use code
rm(list = ls())


#IMPORT ALL THE TABLES FROM AN EXCEL FILE
table1 <- read_xlsx("TidyData.xlsx", sheet = "table1")
table2 <- read_xlsx("TidyData.xlsx", sheet = "table2")
table3 <- read_xlsx("TidyData.xlsx", sheet = "table3")
table4 <- read_xlsx("TidyData.xlsx", sheet = "table4")
table5 <- read_xlsx("TidyData.xlsx", sheet = "table5")


#######################
######TIDY DATA########
#######################
#The principles of tidy data provide a standard way to organise data values within a dataset
#It is often said that 80% of data analysis is spent on the cleaning and preparing data.
#Tidy Data focuses on a small, but important, aspect of data cleaning
#tidying: structuring datasets to facilitate analysis.
#The principles of tidy data provide a standard way to organise data values within a dataset. 
#This makes initial data cleaning easier because you don't need to start from scratch and reinvent the wheel every time. 
#The tidy data standard has been designed to facilitate initial exploration and analysis of the data, and to 
#simplify the development of data analysis tools that work well together.









#Let's return to PowerPoint for a moment











###################
#Tidy Data: table1#
###################
#Tidy data also takes advantage of R's vectorized operations. 
#In R, you often supply one or more vectors of values as input to a function or mathematical operator. 
#Often, the function or operator will use the vectors to create a new vector of values as output,

#Examining the Vectors in table 1
table1$population # a vector
table1$cases # a vector

# people per case
table1$population / table1$cases # a vector of output

#Save the vector
people_per_case <- table1$population / table1$cases
people_per_case

#Add the values to table1
table1
table1$people_per_case <- table1$population / table1$cases
table1


#Hold on! I Have a Question???????
#I just created a new column of data, Do I really need it?
#The answer for now is no.
#Ok, so can I get rid of it?      Great question!
#Removing columns that are not needed --> subset() OR indexing


#Removing a column that is not needed
table1_1 <- subset( table1, select = -people_per_case )#removing a specific column by name --> NOTE: the minus sign
table1_2 <- subset( table1, select =  c(country, year, cases, population))#removing a specific column by name --> NOTE: the minus sign
table1_3 <- table1[,c(1:4)]#indexing --> selecting the columns you want
table1_4 <- table1[,-c(5)]#indexing --> selecting the column(s) you do not

#Let's see if it worked
str(table1_1)
str(table1_2)
str(table1_3)
str(table1_4)
str(table1)





#Back to PowerPoint for a moment








###################
#Tidy Data: table2#
###################
#spread() turns the key:value columns into a set of tidy columns.
#To use spread(), 
#1) pass it the name of a data frame, 
#2) the name of the key column in the data frame, and 
#3) then the name of the value column.
#spread() distributes a pair of key:value columns into a field of cells. The unique keys in the key column become 
#the column names of the field of cells.


#USE THE spread() FUNCTION TO FIX TABLE2
table2
table2_2 <- spread(table2, key, value)
table2_2





##############################
#Tidy Data: table4 and table5#
##############################
#gather() does the reverse of spread(). 
#gather() collects a set of column names and places them into a single “key” column. 
#It also collects the field of cells associated with those columns and places them into a single value column. 
#Just like spread(), gather maintains each of the relationships in the original data set. 


#USE THE gather() FUNCTION TO FIX TABLE4 and TABLE5
#name --> table4
#year --> character string to use for the name of the "key" column that it will male
#cases --> character string to use as the name of the value column it will make
#2:3 --> columns gather() should collapse into the key value pair
table4
table4_1 <- gather(table4, "year", "cases", 2:3)
table4_1

#name --> table5
#year --> character string to use for the name of the "key" column that it will male
#population --> character string to use as the name of the value column it will make
#2:3 --> columns gather() should collapse into the key value pair
table5
table5_1 <- gather(table5, "year", "population", 2:3)
table5_1


#JOINING TABLE4 AND TABLE5
#Joining Options --> inner_join()     left_join()       right_join()     full_join()
#We'll use the left join option for this exercise
table6_1 <- left_join(table4_1, table5_1)# Computer determines join (IF COLUMNS MATCH!!!!)
table6_2 <- left_join(table4_1, table5_1, by = c("country", "year"))# Selecting Columns to join on
table6_1
table6_2



#SEPERATING COLUMNS IN TABLE3
#separate()
#turns a single character column into multiple columns by splitting the values of the column wherever a separator character appears.
#separate(table, column to separate, into = c(“new col1", “new col2"), sep = “separator")
table3
table3_1 <- separate(table3, rate, into = c("cases", "population"), sep = "/")
table3_1
table3_2 <- separate(table3_1, year, into = c("century", "year"), sep = 2)
table3_2


#UNITE TABLE3_2
#unite() 
#does the opposite of separate(): it combines multiple columns into a single column.
#unite(table, “new column to create", col1 to unite, col2 to unite, sep = "")
table3_3 <- unite(table3_2, "year", century, year, sep = "")
table3_3

#Change the name of the tables to final
table1_F <- table1
table2_F <- table2_2
table3_F <- table3_1
table45_F <- table6_2

#TIDY TALBES!!!!!!!!!
table1_F
table2_F
table3_F# notice a difference with this table?
table45_F

#Change the data type for table3_F
table3_F$cases <- as.double(table3_F$cases)
table3_F$population <- as.double(table3_F$population)
table3_F











########################
######ggplot###########
########################
#With ggplot2, you begin a plot with the function ggplot(). 
#ggplot() creates a coordinate system that you can add layers to.
#The first argument of ggplot() is the dataset to use in the graph. 
#You complete your graph by adding one or more layers to ggplot(). 
#ggplot2 comes with many geom functions that each add a different type of layer to a plot
#Each geom function in ggplot2 takes a mapping argument. 
#This defines how variables in your dataset are mapped to visual properties.
#The mapping argument is always paired with aes(), and the x and y arguments of aes() specify which variables to map to the x and y axes. 
#ggplot2 looks for the mapped variable in the data argument

#Let's start by loading the ggplot2 library
library(ggplot2)
library(digest)

#Many R packages come with tables/data
#For our work with ggplot() we will use the "mpg" dataset that comes with the package

#lets look at the data
mpg
str(mpg)
View(mpg)

#We can learn more about the data itself using the help function in R Studio
?mpg


#To plot mpg (dataset), run this code to put "displ" on the x-axis and "hwy" on the y-axis:
ggplot(data = mpg, aes(x = displ, y = hwy)) + geom_point()
#You can see on the right hand side under the "plots" the graph we just created
#You can also export this graph as an image of pdf OR save it to your clipboard
#The plot shows a negative relationship between engine size (displ) and fuel efficiency (hwy). 
#In other words, cars with big engines use more fuel.














##############################
#########Exercise 1############
##############################
#1) Type and Run the following -->  ggplot(data = mpg).      What do you see?


#2) How many rows are in mpg? How many columns?


#3) What does the drv variable describe?


#4) Make a scatterplot of hwy vs cyl.



















#Run the plot we just completed in our example before the exercise
ggplot(data = mpg, aes(x = displ, y = hwy)) + geom_point()

#I'm not a big fan of the basic black color for the points. Let's change this
ggplot(data = mpg, aes(x = displ, y = hwy)) + geom_point(color = "red")


#There are plenty of colours to choose from!!!! --> http://sape.inf.usi.ch/quick-reference/ggplot2/colour


#In the plot above, one group of points seems to fall outside of the linear trend. 
#These cars have a higher mileage than you might expect. How can you explain these cars?
#Could these cars be hybrids?
#One way to test this hypothesis is to look at the class value for each car. 
#The class variable of the mpg dataset classifies cars into groups such as compact, midsize, and SUV. 
#If the outlying points are hybrids, they should be classified as compact cars or, perhaps, subcompact cars
#You can convey information about your data by mapping the aesthetics in your plot to the variables in your dataset.
#For example, you can map different shapes for your points to the class variable to reveal the class of each car.
ggplot(data = mpg, aes(x = displ, y = hwy, color = class)) + geom_point()

#note the color = class added in the aes
#we can see that the points to the right on the graph are all 2seaters --> likely showing that our hypothesis has some merit

#Using sizes for categorization
ggplot(data = mpg, aes(x = displ, y = hwy, size = class)) + 
  geom_point(color = "red")


#Using shape for categorization
ggplot(data = mpg, mapping = aes(x = displ, y = hwy, shape = class)) + 
  geom_point()


#Using color and shape for categorization
ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy, color = class, shape = class))
#Note the warning message in the terminal. If you look at the graph, you can see that one of the "classes" does not have a shape
#Thus, this might not be the best option to use

#Using color and size for categorization
ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy, color = class, size = class))
#Note --> Discrete variable: variables that can only take on a finite number of values

#I like the above graph! But how can I save it
plot1 <- ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy, color = class, size = class))


plot1
#You can also save export the graph --> under the "plots" tab, click on export















#################
###EXERCISE2#####
#################

#1) Run the below code. What has gone wrong with this code? Why are the points not blue? Can you reqrite it to be correct?
ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy, color = "blue"))


#2) Which variables in mpg are categorical? Which variables are continuous?


#3) Map a continuous variable to color, size, and shape.
#color


#size


#shape


#4) What happens if you map the same variable to multiple aesthetics?


#5) What happens if you map an aesthetic to something other than a variable name, like aes(colour = displ < 5)?

















#FACETS###
#Particularly useful for categorical variables is to split your plot into facets, subplots that each display one subset of the data.
#To facet your plot by a single variable, use facet_wrap(). 
#The first argument of facet_wrap() should be a formula, which you create with ~ followed by a variable name.
#Here “formula” is the name of a data structure in R, not a synonym for “equation”.
#The variable that you pass to facet_wrap() should be discrete.
ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy), color = "red") + 
  facet_wrap(~ class, nrow = 2)

#To facet your plot on the combination of two variables, add facet_grid() to your plot call. 
#The first argument of facet_grid() is also a formula. 
#This time the formula should contain two variable names separated by a ~.
ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy), color = "red") + 
  facet_grid(class ~ year)



#Geometric objects###
#A geom is the geometrical object that a plot uses to represent data.
#To change the geom in your plot, change the geom function that you add to ggplot(). 
#For instance, to make the plots above, you can use this code:
# Original
ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy))

# geom_smooth
ggplot(data = mpg) + 
  geom_smooth(mapping = aes(x = displ, y = hwy))

#Every geom function in ggplot2 takes a mapping argument. However, not every aesthetic works with every geom. 
#You could set the shape of a point, but you couldn’t set the “shape” of a line. 
#On the other hand, you could set the linetype of a line. geom_smooth() will draw a different line, with a different linetype, for 
#each unique value of the variable that you map to linetype.
ggplot(data = mpg) + 
  geom_smooth(mapping = aes(x = displ, y = hwy, linetype = drv, color = class))

#We can even add some color into the graph
ggplot(data = mpg) + 
  geom_smooth(mapping = aes(x = displ, y = hwy, linetype = drv, color = class))

#We can even display multiple geoms in the same plot.
#All we have to do is add multiple geom functions to ggplot()
ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy)) +
  geom_smooth(mapping = aes(x = displ, y = hwy))

#This, however, introduces some duplication in our code. 
#Imagine if you wanted to change the y-axis to display cty instead of hwy. 
#You'd need to change the variable in two places, and you might forget to update one. 
#You can avoid this type of repetition by passing a set of mappings to ggplot()
ggplot(data = mpg, mapping = aes(x = displ, y = hwy)) + 
  geom_point() + 
  geom_smooth()


#If you place mappings in a geom function, ggplot2 will treat them as local mappings for the layer. 
#It will use these mappings to extend or overwrite the global mappings for that layer only. 
#This makes it possible to display different aesthetics in different layers.
ggplot(data = mpg, mapping = aes(x = displ, y = hwy)) + 
  geom_point(mapping = aes(color = class)) + 
  geom_smooth()

#You can use the same idea to specify different data for each layer. 
#Here, our smooth line displays just a subset of the mpg dataset, the subcompact cars. 
#The local data argument in geom_smooth() overrides the global data argument in ggplot() for that layer only.
ggplot(data = mpg, mapping = aes(x = displ, y = hwy)) + 
  geom_point(mapping = aes(color = class)) + 
  geom_smooth(data = filter(mpg, class == "subcompact"), se = FALSE)
#NOTE: se = Display confidence interval around smooth? (TRUE by default, see level to control.)
















#####################
#####EXERCISE3#######
#####################
#1) Create a graph from the mp3 dataset that uses both geom_point() and geom_smooth with:
#x = displ
#y = hwy
#use the color black for the line
#categorize the points using the "class" attribute using the "color" option
#Assign the graph the name "my_ggplot"




















############################
#######BAR GRAPHS###########
############################
#Lets look at geom_bar() using the diamonds dataset (found in the ggplot package)
?diamonds
diamonds
str(diamonds)

#Bar charts seem simple, but they are interesting because they reveal something subtle about plots.
#Consider a basic bar chart, as drawn with geom_bar(). 
#The following chart displays the total number of diamonds in the diamonds dataset, grouped by cut. 
ggplot(data = diamonds) + 
  geom_bar(mapping = aes(x = cut))
#On the x-axis, the chart displays cut, a variable from diamonds. 
#On the y-axis, it displays count, but count is not a variable in diamonds! 
#Where does count come from? Many graphs, like scatterplots, plot the raw values of your dataset.
#Other graphs, like bar charts, calculate new values to plot:
#Bar charts, histograms, and frequency polygons bin your data and then plot bin counts, the number of points that fall in each bin.
#You can learn which stat a geom uses by inspecting the default value for the stat argument.
?geom_bar()
#we can see that geom_bar uses "stat_count" to create the numeric value of "count" in the plot
#We can gain the same results at the previous bar chart by substituting geom_bar for stat_count
ggplot(data = diamonds) + 
  stat_count(mapping = aes(x = cut))

##########################################
#Position Adjustment with geom_bar()######
##########################################
#You can colour a bar chart using either the colour aesthetic, or, more usefully, fill
#This one colors the outline of each bar
ggplot(data = diamonds) + 
  geom_bar(mapping = aes(x = cut, colour = cut))

#This one changes the fill color of the bar chart
ggplot(data = diamonds) + 
  geom_bar(mapping = aes(x = cut, fill = cut))

#Note what happens if you map the fill aesthetic to another variable, like clarity: 
#the bars are automatically stacked. 
#Each colored rectangle represents a combination of cut and clarity
ggplot(data = diamonds) + 
  geom_bar(mapping = aes(x = cut, fill = clarity))

#The stacking is performed automatically by the position adjustment specified by the position argument. 
#If you don't want a stacked bar chart, you can use one of three other options: "identity", "dodge" or "fill"

#position = "identity" will place each object exactly where it falls in the context of the graph. 
#This is not very useful for bars, because it overlaps them. 
#To see that overlapping we either need to make the bars slightly transparent by setting alpha to a small value,
#or completely transparent by setting fill = NA.

#This options will shade the bar colors
ggplot(data = diamonds, mapping = aes(x = cut, fill = clarity)) + 
  geom_bar(alpha = 1/5, position = "identity")
#This option will remove the fill
ggplot(data = diamonds, mapping = aes(x = cut, colour = clarity)) + 
  geom_bar(fill = NA, position = "identity")

#position = "fill" works like stacking, but makes each set of stacked bars the same height.
#This makes it easier to compare proportions across groups.
ggplot(data = diamonds) + 
  geom_bar(mapping = aes(x = cut, fill = clarity), position = "fill")

#position = "dodge" places overlapping objects directly beside one another. 
#This makes it easier to compare individual values.
ggplot(data = diamonds) + 
  geom_bar(mapping = aes(x = cut, fill = clarity), position = "dodge")












##################
#####EXERCISE4####
##################
#1) Use the geom_bar() function to plot the following
#x = clarity
#fill = color
#Build three different graph using the "position" options (dodge, identity, and fill)




















#####################################
#SOME REALLY COOL STUFF!!!!!
#####################################
###What you can do with ggplot()#####
#####################################
#Some of you may be saying, "Ok, all the things you have shown me I can do with some software that I already use.
#What is the big deal with using R to do these things? 
#Remember, I am just showing you some basics. With more work and consultation of available resouces, you can do some really cool things.
#HERE ARE SOME EXAMPLES




###########################################
######Correlation and Correlogram##########
###########################################
mtcars# use the mtcars data set provided in our pre-loaded R package
str(mtcars)

#simple correlation
cor(mtcars$mpg, mtcars$wt)# correlation between mpg and wt

cor(mtcars)#all correlations between all numeric variables

cars_cor <- cor(mtcars)# save the above correlations as an object

View(cars_cor)#view the correlation table

#Both of the methods above make it quite difficult to assess all the correlations.
#Is there a better way? YES!
install.packages("ggcorrplot")
#install.packages("ggplot2")
library(ggplot2)
library(ggcorrplot)

# Correlation matrix
data(mtcars)#bring in the mtcars dataset
corr <- round(cor(mtcars), 1)#round the correlations
View(corr)

# Plot
ggcorrplot(corr, hc.order = TRUE, 
           type = "lower", 
           lab = TRUE, 
           lab_size = 3, 
           method="circle", 
           colors = c("tomato2", "white", "springgreen3"), 
           title="Correlogram of mtcars", 
           ggtheme=theme_bw)
#This is a much better way to assess the correations as the visualization provides both the correlation value, along with a coloring 
#scheme for easy asessment


####YAHOO STOCK PRICE#####
# http://margintale.blogspot.in/2012/04/ggplot2-time-series-heatmaps.html
library(ggplot2)
install.packages("plyr")
library(plyr)
install.packages("scales")
library(scales)
install.packages("zoo")
library(zoo)

df <- read.csv("https://raw.githubusercontent.com/selva86/datasets/master/yahoo.csv")
View(df)
str(df)

df$date <- as.Date(df$date)  # format date
str(df)

nrow(df)# number of rows
df <- df[df$year >= 2012, ]  # filter required years
nrow(df)# number of rows

# Create Month Week
df$yearmonth <- as.yearmon(df$date)
df$yearmonthf <- factor(df$yearmonth)#convert to factor
df <- ddply(df,.(yearmonthf), transform, monthweek=1+week-min(week))  # compute week number of month
df <- df[, c("year", "yearmonthf", "monthf", "week", "monthweek", "weekdayf", "VIX.Close")]#select needed columns
head(df)

# Plot
ggplot(df, aes(monthweek, weekdayf, fill = VIX.Close)) + 
  geom_tile(colour = "white") + 
  facet_grid(year~monthf) + 
  scale_fill_gradient(low="red", high="green") +
  labs(x="Week of Month",
       y="",
       title = "Time-Series Calendar Heatmap", 
       subtitle="Yahoo Closing Price", 
       fill="Close")











#QUICK DISCUSSION REGARDING RMARKDOWN
#I brought this up earlier with regards to my "Reasons to learn R for business slide"


#Let's try it out and see if we can replicate the code for create some maps!
#Let's create two things:
#Interactive bubble map with plotly
#Basic scatterplot map
#https://www.r-graph-gallery.com/330-bubble-map-with-ggplot2.html




#Here is a bit of information to get you started if you are interested in trying it --> https://rmarkdown.rstudio.com/articles_intro.html








##############
##dplyr#######
##############
#What is dplyr?
#dplyr is a powerful R-package to transform and summarize tabular data with rows and columns. 


#Why is it useful?
#Great for data exploration and transformation
#Intuitive to write and easy to read, especially when using the “chaining” syntax (covered below)
#Fast on data frames

#dplyr functionality
#Five basic verbs: filter, select, arrange, mutate, summarise (plus group_by)
#Can work with data stored in databases and data tables
#Joins: inner join, left join, semi-join, anti-join (not covered below)
#Window functions for calculating ranking, offsets, and more


#Let's Install the dplyr package and load the library
install.packages("dplyr")#install dplyr
library(dplyr)# load dplyr package

#For our tutorial, we will be using the dataset hflights that is found in the hflights package
#Let's install the package and load the library
install.packages("hflights")
library(hflights)

#Let's take a look at our data
str(hflights)
head(hflights)

#To make our lives a bit easier, let's convert hflights to a local data frame
flights <- tbl_df(hflights)#tbl_df creates a “local data frame”. Local data frame is simply a wrapper for a data frame that prints nicely
str(flights)
View(flights)


##########filter()##########
#Base R approach to filtering forces you to repeat the data frame’s name
#dplyr approach is simpler to write and read

#base R approach to view all flights on January 1
flights[flights$Month==1 & flights$DayofMonth==1, ]

#dplyr approach. # note: you can use comma or ampersand to represent AND condition
filter(flights, Month==1, DayofMonth==1)

# use pipe for OR condition
filter(flights, UniqueCarrier=="AA" | UniqueCarrier=="UA")

# you can also use %in% operator
filter(flights, UniqueCarrier %in% c("AA", "UA"))
str(flights)


#########select()##########
#Pick columns by name
#Base R approach is awkward to type and to read
#dplyr approach uses similar syntax to filter
#Like a SELECT in SQL

# base R approach to select DepTime, ArrTime, and FlightNum columns
flights[, c("DepTime", "ArrTime", "FlightNum")]

# dplyr approach
select(flights, DepTime, ArrTime, FlightNum)


# use colon to select multiple contiguous columns, and use `contains` to match columns by name
# note: `starts_with`, `ends_with`, and `matches` (for regular expressions) can also be used to match columns by name

#select columns from Year to DayofMonth
select(flights, Year:DayofMonth)

#select columns that contains the word "Taxi"
select(flights, contains("Taxi"))

#select columns that contains the word "Delay"
select(flights, ends_with("Delay"))

#Combination of the three scripts above
select(flights, Year:DayofMonth, contains("Taxi"), ends_with("Delay"))

#We can also use the "-" tell the function which columns not to include
select(flights, -Year:-DayofMonth)


#####A BRIEF ASIDE####
#Ok, once again, you may be thinking what is the big deal here? These are just two different ways of doing the same thing when it comes
#to base R vs. dplyr. What is the big deal?
#Enter my favourite operator when using R!!!!!!!!!!


######“CHAINING” or “PIPELINING”#######
#Usual way to perform multiple operations in one line is by nesting
#Can write commands in a natural order by using the %>% infix operator (which can be pronounced as “then”)

# nesting method to select UniqueCarrier and DepDelay columns and filter for delays over 60 minutes
filter(select(flights, UniqueCarrier, DepDelay), DepDelay > 60)

# chaining method
flights %>%
  select(UniqueCarrier, DepDelay) %>%
  filter(DepDelay > 60)

#So, what is the big deal?
#Chaining increases readability significantly when there are many commands
#Operator is automatically imported from the magrittr package
#Can be used to replace nesting in R commands outside of dplyr


#####arrange()#####
#Re-ordering columns

# base R approach to select UniqueCarrier and DepDelay columns and sort by DepDelay
flights[order(flights$DepDelay), c("UniqueCarrier", "DepDelay")]

# dplyr approach
flights %>%
  select(UniqueCarrier, DepDelay) %>%
  arrange(DepDelay)

# use `desc` for descending
flights %>%
  select(UniqueCarrier, DepDelay) %>%
  arrange(desc(DepDelay))


#####mutate()######
#Add new variables

#we will use a duplicate copy of the flights dataset for this exercise
flights1 <- flights

# base R approach to create a new variable Speed (in mph)
flights1$Speed <- flights$Distance / flights$AirTime*60
flights1[, c("Distance", "AirTime", "Speed")]

# dplyr approach (prints the new variable but does not store it)
flights1 %>%
  select(Distance, AirTime) %>%
  mutate(Speed = Distance/AirTime*60)


#####summarise() and group_by()#########
#Primarily useful with data that has been grouped by one or more variables
#group_by creates the groups that will be operated on
#summarise uses the provided aggregation function to summarise each group

# base R approaches to calculate the average arrival delay to each destination
head(with(flights, tapply(ArrDelay, Dest, mean, na.rm=TRUE)))
head(aggregate(ArrDelay ~ Dest, flights, mean))

# dplyr approach: create a table grouped by Dest, and then summarise each group by taking the mean of ArrDelay
flights %>%
  select(Dest, ArrDelay) %>%
  group_by (Dest) %>%
  summarise(mean = mean(ArrDelay, na.rm=TRUE))

#A BRIEF ASIDE
#You'll notice in the above code that the term "na.rm = TRUE" was used. This is added in our code
#to remove missing values from our calculations. 
#We'll come back to missing values and dealing with them in R later on.

#summarise_each() allows you to apply the same summary function to multiple columns at once
#Note: mutate_each is also available

#for each carrier, calculate the percentage of flights cancelled or diverted
flights %>%
  group_by(UniqueCarrier) %>%
  summarise_each(funs(mean), Cancelled, Diverted)

#for each carrier, calculate the minimum and maximum arrival and departure delays
flights %>%
  group_by(UniqueCarrier) %>%
  summarise_each(funs(min(., na.rm=TRUE), max(., na.rm=TRUE)), matches("Delay"))

#for each day of the year, count the total number of flights and sort in descending order
flights %>%
  group_by(Month, DayofMonth) %>%
  tally(sort = TRUE)

#In some instances, grouping can sometimes be useful without summarising
#Here is an example. For each destination, show the number of cancelled and not cancelled flights
flights %>%
  group_by(Dest) %>%
  select(Cancelled) %>%
  table() %>%
  head()


















#############################
#########EXERCISE5###########
#############################

#1) select the columns Month, DayofWeek, Origin, Dest, Destination, Distance, and AirTime. Save the data as an object called "my_flights"


#2) From the created "my_flights" table, use the filter function to include only those flights that occurred in November.
#Save the data as an object called "my_flights_2"



#3) From the created "my_flights2" table, create a new column called "Speed" using the mutate() function. Save the data as an object called "my_flights_3"
#Hint: Speed is calculated as Distance divided by Airtime times 60 minutes




#4)From the created "my_flights3" table, create a new table that provides the average speed and average airtime for each destination.
#Save the data as an object caleed "my_flights4"
#Hint: you will need to use the group_by() and summarise() functions.




#5) Using the flights dataset, create an object called "my_flights5" that combines steps 1 through 4













###########################################
#Dealing with NA values --> missing values#
###########################################
#For this exercise, we will use the "Medical Cost Personal Datasets_Insurance_NA.xlsx" found in the data provided to you
#Let's Import the data
library(readxl)
setwd("C:\\Users\\eg61211\\OneDrive - Co-operators\\Desktop\\Brock\\Datathon - R Workshop\\datasets")
Medical_Insurance_NA <- read_xlsx("Medical Cost Personal Datasets_Insurance_NA.xlsx", sheet = 1)#sheet by name
str(Medical_Insurance_NA)
View(Medical_Insurance_NA)


# We can examine our data visually to determine if there are any missing values
#This can be done through a variety of packages
#AMELIA PACKAGE
install.packages("Amelia")
library(Amelia)
missmap(Medical_Insurance_NA)

#NANIAR PACKAGE
install.packages("naniar")
library(naniar)
vis_miss(Medical_Insurance_NA)

#UPSETR PACKAGE --> my favourite!
install.packages("UpSetR")
library(UpSetR)
library(naniar)
gg_miss_upset(Medical_Insurance_NA)
#This tells us:
#1) Only Age and BMI have missing values
#2) BMI has the most missing values
#3) There is 1 cases where both Age and BMI have missing values together

#Here are some function that provide quick summaries of missingness in your data, they all start with gg_miss_ - so that they are easy to remember and tab-complete.
library(ggplot2)
library(tidyr)
gg_miss_var(Medical_Insurance_NA)
gg_miss_var(airquality) + labs(y = "Look at all the missing ones")#add a label/title

#Facets --> remember this topic from the ggplot section?
#You can also plot the number of missings in a variable grouped by another variable using the facet argument.
gg_miss_var(Medical_Insurance_NA,
            facet = region)

#We can also have some fun with dplyr in examining the missing values
Medical_Insurance_NA %>%
  group_by(region) %>%
  miss_var_summary()# this function is part of the "UpSetR" package loaded above
#It is interesting to note that with dplyr, we can combine functions from other packages and chain them together to make our coding more efficient

#So, we have missing values, how can we impute them?
#Let's try it out with dplyr - a more hands on approach.
#We can impute using central tendency (mean/median/mode) for both bmi and age 
#Note: we will use mean for this example
Medical_Insurance_NA_mean_impute <- Medical_Insurance_NA %>%
  mutate(missing_bmi = ifelse(is.na(bmi), 1,0),
         missing_age = ifelse(is.na(age), 1,0),
         bmi = replace_na(bmi, mean(bmi, na.rm = TRUE)),
         age = replace_na(age, mean(age, na.rm = TRUE)),
         age = format(round(age, 0), nsmall = 0))
View(Medical_Insurance_NA_mean_impute)

#MICE PACKAGE OPTION
#MICE (Multivariate Imputation via Chained Equations) is one of the commonly used package by R users. 
#Creating multiple imputations as compared to a single imputation (such as mean) takes care of uncertainty in missing values.
#MICE assumes that the missing data are Missing at Random (MAR), which means that the probability that a value is missing depends only on observed value 
#and can be predicted using them. It imputes data on a variable by variable basis by specifying an imputation model per variable.
install.packages("mice")
library(mice)

#The Methods used by the MICE package are:
#1) PMM (Predictive Mean Matching) --> For numeric variables
#2) logreg(Logistic Regression)--> For Binary Variables( with 2 levels)
#3) polyreg(Bayesian polytomous regression)--> For Factor Variables (>= 2 levels)
#4) Proportional odds model (ordered, >= 2 levels)

#let’s impute the missing values.
#Here is an explanation of the parameters used:
#m --> Refers to 1 imputed data sets
#maxit --> Refers to no. of iterations taken to impute missing values
#method --> Refers to method used in imputation. we used predictive mean matching.
imputed_Data <- mice(Medical_Insurance_NA, m=1, maxit = 200, method = 'pmm', seed = 123)
#IMPORTANT --< SEED!!!!!! This allows you to replicate your results if run again in the future
summary(imputed_Data)# summary of the imputation process

#We can also check our imputted values
imputed_Data$imp$bmi# bmi imputted values
imputed_Data$imp$age# age imputted values

#Let's take the imputted dataset and save it with a new name
Medical_Insurance_complete <- complete(imputed_Data)
View(Medical_Insurance_complete)













#####################
#EXPORTING YOUR DATA#
#####################
#Ok, we've done a lot of analysis and created a bunch of new data frames and objects.
#The last thing that we need to discuss is how to we get this data out of R.
#There are going to be many instances where your data is going to have to be exported
#and provided to other people in a format that they are familiar with.



?write.table()
#In examining the write.table() function, we can see the various arguments that can be used
#when exporting your data

#First thing you should always do is set your working directory (the directory we want to export our data to)
setwd("C:\\Users\\pfrosina\\Documents\\R Workshop\\Exported Data")

#Let's export our Medical_Insurance_complete data to txt file with tab separated values
write.table(Medical_Insurance_complete, file = "Medical_Insurance_complete_tab.txt", sep = "\t",
            row.names = TRUE, col.names = TRUE)
#the sep = "\t" operator tells R to export it with tabs seperating our values

#What if we wanted to export a text file with a ";" seperating our data?
write.table(Medical_Insurance_complete, file = "Medical_Insurance_complete_semi.txt", sep = ";",
            row.names = TRUE, col.names = TRUE)

#Let's look at our data in the folder
#You'll notice that there are more columns than variables. Why did this happen?
#When you use the argument "row.names - TRUE" or don't include this argument, 
#R will export the row numbers in the data frame with your data.
#We don't want this, so let's exclude the row name.
#NOTE: make sure to close the files we just exported before running the below code.
write.table(Medical_Insurance_complete, file = "Medical_Insurance_complete_tab.txt", sep = "\t",
            row.names = FALSE, col.names = TRUE)
write.table(Medical_Insurance_complete, file = "Medical_Insurance_complete_semi.txt", sep = ";",
            row.names = FALSE, col.names = TRUE)

# Write data to csv files:  
write.csv(Medical_Insurance_complete, file = "Medical_Insurance_complete.csv", row.names = FALSE)

#Write data to an excel file
library(xlsx)# NOTE: this may not work on the school computers due to issues with Java.
write.xlsx(Medical_Insurance_complete, "Medical_Insurance_complete.xlsx", sheetName = "Sheet1", append = FALSE) 

## write out text datafile and an SPSS program to read it
library(haven)
write_sav(Medical_Insurance_complete, "Medical_Insurance_complete_SPSS.sav")

# write out text datafile and a SAS program to read it
library(foreign)
write.foreign(Medical_Insurance_complete, "Medical_Insurance_complete_SAS.txt", "Medical_Insurance_complete.sas",   package="SAS") 





#References for this workshop
#https://cran.r-project.org/web/packages/naniar/vignettes/naniar-visualisation.html
#http://www.sthda.com/english/wiki/exporting-data-from-r
#https://datascienceplus.com/data-manipulation-with-dplyr/
#http://sachaepskamp.com/files/dplyrTutorial.html#filter
#http://genomicsclass.github.io/book/pages/dplyr_tutorial.html
#http://rpubs.com/justmarkham/dplyr-tutorial
#https://r4ds.had.co.nz/index.html
#http://r-statistics.co/Top50-Ggplot2-Visualizations-MasterList-R-Code.html
